@@ -1,21 +1,27 @@
-from __future__ import unicode_literals
-
 import os
 import sys
 import string
 import platform
-
-from pkg_resources.extern.six.moves import map
+import itertools
 
 import pytest
 from pkg_resources.extern import packaging
 
 import pkg_resources
-from pkg_resources import (parse_requirements, VersionConflict, parse_version,
+from pkg_resources import (
+    parse_requirements, VersionConflict, parse_version,
     Distribution, EntryPoint, Requirement, safe_version, safe_name,
     WorkingSet)
 
 
+# from Python 3.6 docs.
+def pairwise(iterable):
+    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
+    a, b = itertools.tee(iterable)
+    next(b, None)
+    return zip(a, b)
+
+
 class Metadata(pkg_resources.EmptyProvider):
     """Mock object to return metadata as if from an on-disk distribution"""
 
@@ -36,7 +42,6 @@ dist_from_fn = pkg_resources.Distribution.from_filename
 
 
 class TestDistro:
-
     def testCollection(self):
         # empty path should produce no distributions
         ad = pkg_resources.Environment([], platform=None, python=None)
@@ -52,7 +57,8 @@ class TestDistro:
         assert list(ad) == ['foopkg']
 
         # Distributions sort by version
-        assert [dist.version for dist in ad['FooPkg']] == ['1.4', '1.3-1', '1.2']
+        expected = ['1.4', '1.3-1', '1.2']
+        assert [dist.version for dist in ad['FooPkg']] == expected
 
         # Removing a distribution leaves sequence alone
         ad.remove(ad['FooPkg'][1])
@@ -98,12 +104,15 @@ class TestDistro:
     def testDistroBasics(self):
         d = Distribution(
             "/some/path",
-            project_name="FooPkg", version="1.3-1", py_version="2.4", platform="win32"
+            project_name="FooPkg",
+            version="1.3-1",
+            py_version="2.4",
+            platform="win32",
         )
         self.checkFooPkg(d)
 
         d = Distribution("/some/path")
-        assert d.py_version == sys.version[:3]
+        assert d.py_version == '{}.{}'.format(*sys.version_info)
         assert d.platform is None
 
     def testDistroParse(self):
@@ -114,10 +123,11 @@ class TestDistro:
 
     def testDistroMetadata(self):
         d = Distribution(
-            "/some/path", project_name="FooPkg", py_version="2.4", platform="win32",
+            "/some/path", project_name="FooPkg",
+            py_version="2.4", platform="win32",
             metadata=Metadata(
                 ('PKG-INFO', "Metadata-Version: 1.0\nVersion: 1.3-1\n")
-            )
+            ),
         )
         self.checkFooPkg(d)
 
@@ -131,6 +141,35 @@ class TestDistro:
         for v in "Twisted>=1.5", "Twisted>=1.5\nZConfig>=2.0":
             self.checkRequires(self.distRequires(v), v)
 
+    needs_object_dir = pytest.mark.skipif(
+        not hasattr(object, '__dir__'),
+        reason='object.__dir__ necessary for self.__dir__ implementation',
+    )
+
+    def test_distribution_dir(self):
+        d = pkg_resources.Distribution()
+        dir(d)
+
+    @needs_object_dir
+    def test_distribution_dir_includes_provider_dir(self):
+        d = pkg_resources.Distribution()
+        before = d.__dir__()
+        assert 'test_attr' not in before
+        d._provider.test_attr = None
+        after = d.__dir__()
+        assert len(after) == len(before) + 1
+        assert 'test_attr' in after
+
+    @needs_object_dir
+    def test_distribution_dir_ignores_provider_dir_leading_underscore(self):
+        d = pkg_resources.Distribution()
+        before = d.__dir__()
+        assert '_test_attr' not in before
+        d._provider._test_attr = None
+        after = d.__dir__()
+        assert len(after) == len(before)
+        assert '_test_attr' not in after
+
     def testResolve(self):
         ad = pkg_resources.Environment([])
         ws = WorkingSet([])
@@ -165,7 +204,10 @@ class TestDistro:
         ad.add(Baz)
 
         # Activation list now includes resolved dependency
-        assert list(ws.resolve(parse_requirements("Foo[bar]"), ad)) == [Foo, Baz]
+        assert (
+            list(ws.resolve(parse_requirements("Foo[bar]"), ad))
+            == [Foo, Baz]
+        )
         # Requests for conflicting versions produce VersionConflict
         with pytest.raises(VersionConflict) as vc:
             ws.resolve(parse_requirements("Foo==1.2\nFoo!=1.2"), ad)
@@ -207,12 +249,10 @@ class TestDistro:
         """Extras are also evaluated as markers at resolution time."""
         ad = pkg_resources.Environment([])
         ws = WorkingSet([])
-        # Metadata needs to be native strings due to cStringIO behaviour in
-        # 2.6, so use str().
         Foo = Distribution.from_filename(
             "/foo_dir/Foo-1.2.dist-info",
-            metadata=Metadata(("METADATA", str("Provides-Extra: baz\n"
-                               "Requires-Dist: quux; extra=='baz'")))
+            metadata=Metadata(("METADATA", "Provides-Extra: baz\n"
+                               "Requires-Dist: quux; extra=='baz'"))
         )
         ad.add(Foo)
         assert list(ws.resolve(parse_requirements("Foo"), ad)) == [Foo]
@@ -221,17 +261,31 @@ class TestDistro:
         res = list(ws.resolve(parse_requirements("Foo[baz]"), ad))
         assert res == [Foo, quux]
 
+    def test_marker_evaluation_with_extras_normlized(self):
+        """Extras are also evaluated as markers at resolution time."""
+        ad = pkg_resources.Environment([])
+        ws = WorkingSet([])
+        Foo = Distribution.from_filename(
+            "/foo_dir/Foo-1.2.dist-info",
+            metadata=Metadata(("METADATA", "Provides-Extra: baz-lightyear\n"
+                               "Requires-Dist: quux; extra=='baz-lightyear'"))
+        )
+        ad.add(Foo)
+        assert list(ws.resolve(parse_requirements("Foo"), ad)) == [Foo]
+        quux = Distribution.from_filename("/foo_dir/quux-1.0.dist-info")
+        ad.add(quux)
+        res = list(ws.resolve(parse_requirements("Foo[baz-lightyear]"), ad))
+        assert res == [Foo, quux]
+
     def test_marker_evaluation_with_multiple_extras(self):
         ad = pkg_resources.Environment([])
         ws = WorkingSet([])
-        # Metadata needs to be native strings due to cStringIO behaviour in
-        # 2.6, so use str().
         Foo = Distribution.from_filename(
             "/foo_dir/Foo-1.2.dist-info",
-            metadata=Metadata(("METADATA", str("Provides-Extra: baz\n"
+            metadata=Metadata(("METADATA", "Provides-Extra: baz\n"
                                "Requires-Dist: quux; extra=='baz'\n"
                                "Provides-Extra: bar\n"
-                               "Requires-Dist: fred; extra=='bar'\n")))
+                               "Requires-Dist: fred; extra=='bar'\n"))
         )
         ad.add(Foo)
         quux = Distribution.from_filename("/foo_dir/quux-1.0.dist-info")
@@ -244,22 +298,20 @@ class TestDistro:
     def test_marker_evaluation_with_extras_loop(self):
         ad = pkg_resources.Environment([])
         ws = WorkingSet([])
-        # Metadata needs to be native strings due to cStringIO behaviour in
-        # 2.6, so use str().
         a = Distribution.from_filename(
             "/foo_dir/a-0.2.dist-info",
-            metadata=Metadata(("METADATA", str("Requires-Dist: c[a]")))
+            metadata=Metadata(("METADATA", "Requires-Dist: c[a]"))
         )
         b = Distribution.from_filename(
             "/foo_dir/b-0.3.dist-info",
-            metadata=Metadata(("METADATA", str("Requires-Dist: c[b]")))
+            metadata=Metadata(("METADATA", "Requires-Dist: c[b]"))
         )
         c = Distribution.from_filename(
             "/foo_dir/c-1.0.dist-info",
-            metadata=Metadata(("METADATA", str("Provides-Extra: a\n"
+            metadata=Metadata(("METADATA", "Provides-Extra: a\n"
                                "Requires-Dist: b;extra=='a'\n"
                                "Provides-Extra: b\n"
-                               "Requires-Dist: foo;extra=='b'")))
+                               "Requires-Dist: foo;extra=='b'"))
         )
         foo = Distribution.from_filename("/foo_dir/foo-0.1.dist-info")
         for dist in (a, b, c, foo):
@@ -267,6 +319,10 @@ class TestDistro:
         res = list(ws.resolve(parse_requirements("a"), ad))
         assert res == [a, c, b, foo]
 
+    @pytest.mark.xfail(
+        sys.version_info[:2] == (3, 12) and sys.version_info.releaselevel != 'final',
+        reason="https://github.com/python/cpython/issues/103632",
+    )
     def testDistroDependsOptions(self):
         d = self.distRequires("""
             Twisted>=1.5
@@ -295,7 +351,6 @@ class TestDistro:
 
 
 class TestWorkingSet:
-
     def test_find_conflicting(self):
         ws = WorkingSet([])
         Foo = Distribution.from_filename("/foo_dir/Foo-1.2.egg")
@@ -338,7 +393,6 @@ class TestWorkingSet:
 
 
 class TestEntryPoints:
-
     def assertfields(self, ep):
         assert ep.name == "foo"
         assert ep.module_name == "pkg_resources.tests.test_resources"
@@ -408,7 +462,8 @@ class TestEntryPoints:
 
     submap_expect = dict(
         feature1=EntryPoint('feature1', 'somemodule', ['somefunction']),
-        feature2=EntryPoint('feature2', 'another.module', ['SomeClass'], ['extra1', 'extra2']),
+        feature2=EntryPoint(
+            'feature2', 'another.module', ['SomeClass'], ['extra1', 'extra2']),
         feature3=EntryPoint('feature3', 'this.module', extras=['something'])
     )
     submap_str = """
@@ -437,9 +492,16 @@ class TestEntryPoints:
         with pytest.raises(ValueError):
             EntryPoint.parse_map(self.submap_str)
 
+    def testDeprecationWarnings(self):
+        ep = EntryPoint(
+            "foo", "pkg_resources.tests.test_resources", ["TestEntryPoints"],
+            ["x"]
+        )
+        with pytest.warns(pkg_resources.PkgResourcesDeprecationWarning):
+            ep.load(require=False)
+
 
 class TestRequirements:
-
     def testBasics(self):
         r = Requirement.parse("Twisted>=1.2")
         assert str(r) == "Twisted>=1.2"
@@ -457,6 +519,11 @@ class TestRequirements:
         assert r1 == r2
         assert str(r1) == str(r2)
         assert str(r2) == "Twisted==1.2c1,>=1.2"
+        assert (
+            Requirement("Twisted")
+            !=
+            Requirement("Twisted @ https://localhost/twisted.zip")
+        )
 
     def testBasicContains(self):
         r = Requirement("Twisted>=1.2")
@@ -483,11 +550,23 @@ class TestRequirements:
             ==
             hash((
                 "twisted",
+                None,
                 packaging.specifiers.SpecifierSet(">=1.2"),
                 frozenset(["foo", "bar"]),
                 None
             ))
         )
+        assert (
+            hash(Requirement.parse("Twisted @ https://localhost/twisted.zip"))
+            ==
+            hash((
+                "twisted",
+                "https://localhost/twisted.zip",
+                packaging.specifiers.SpecifierSet(),
+                frozenset(),
+                None
+            ))
+        )
 
     def testVersionEquality(self):
         r1 = Requirement.parse("foo==0.3a2")
@@ -512,15 +591,20 @@ class TestRequirements:
             Requirement.parse('setuptools').project_name == 'setuptools')
         # setuptools 0.7 and higher means setuptools.
         assert (
-            Requirement.parse('setuptools == 0.7').project_name == 'setuptools')
+            Requirement.parse('setuptools == 0.7').project_name
+            == 'setuptools'
+        )
         assert (
-            Requirement.parse('setuptools == 0.7a1').project_name == 'setuptools')
+            Requirement.parse('setuptools == 0.7a1').project_name
+            == 'setuptools'
+        )
         assert (
-            Requirement.parse('setuptools >= 0.7').project_name == 'setuptools')
+            Requirement.parse('setuptools >= 0.7').project_name
+            == 'setuptools'
+        )
 
 
 class TestParsing:
-
     def testEmptyParse(self):
         assert list(parse_requirements('')) == []
 
@@ -547,7 +631,7 @@ class TestParsing:
                     """
         assert (
             list(pkg_resources.split_sections(sample))
-                ==
+            ==
             [
                 (None, ["x"]),
                 ("Y", ["z", "a"]),
@@ -580,7 +664,7 @@ class TestParsing:
             [Requirement('Twis-Ted>=1.2-1')]
         )
         assert (
-            list(parse_requirements('Twisted >=1.2, \ # more\n<2.0'))
+            list(parse_requirements('Twisted >=1.2, \\ # more\n<2.0'))
             ==
             [Requirement('Twisted>=1.2,<2.0')]
         )
@@ -609,7 +693,7 @@ class TestParsing:
         assert (
             Requirement.parse("name==1.1;python_version=='2.7'")
             !=
-            Requirement.parse("name==1.1;python_version=='3.3'")
+            Requirement.parse("name==1.1;python_version=='3.6'")
         )
         assert (
             Requirement.parse("name==1.0;python_version=='2.7'")
@@ -617,105 +701,65 @@ class TestParsing:
             Requirement.parse("name==1.2;python_version=='2.7'")
         )
         assert (
-            Requirement.parse("name[foo]==1.0;python_version=='3.3'")
+            Requirement.parse("name[foo]==1.0;python_version=='3.6'")
             !=
-            Requirement.parse("name[foo,bar]==1.0;python_version=='3.3'")
+            Requirement.parse("name[foo,bar]==1.0;python_version=='3.6'")
         )
 
     def test_local_version(self):
-        req, = parse_requirements('foo==1.0.org1')
+        req, = parse_requirements('foo==1.0+org1')
 
     def test_spaces_between_multiple_versions(self):
         req, = parse_requirements('foo>=1.0, <3')
         req, = parse_requirements('foo >= 1.0, < 3')
 
-    def testVersionEquality(self):
-        def c(s1, s2):
-            p1, p2 = parse_version(s1), parse_version(s2)
-            assert p1 == p2, (s1, s2, p1, p2)
+    @pytest.mark.parametrize(
+        ['lower', 'upper'],
+        [
+            ('1.2-rc1', '1.2rc1'),
+            ('0.4', '0.4.0'),
+            ('0.4.0.0', '0.4.0'),
+            ('0.4.0-0', '0.4-0'),
+            ('0post1', '0.0post1'),
+            ('0pre1', '0.0c1'),
+            ('0.0.0preview1', '0c1'),
+            ('0.0c1', '0-rc1'),
+            ('1.2a1', '1.2.a.1'),
+            ('1.2.a', '1.2a'),
+        ],
+    )
+    def testVersionEquality(self, lower, upper):
+        assert parse_version(lower) == parse_version(upper)
 
-        c('1.2-rc1', '1.2rc1')
-        c('0.4', '0.4.0')
-        c('0.4.0.0', '0.4.0')
-        c('0.4.0-0', '0.4-0')
-        c('0post1', '0.0post1')
-        c('0pre1', '0.0c1')
-        c('0.0.0preview1', '0c1')
-        c('0.0c1', '0-rc1')
-        c('1.2a1', '1.2.a.1')
-        c('1.2.a', '1.2a')
-
-    def testVersionOrdering(self):
-        def c(s1, s2):
-            p1, p2 = parse_version(s1), parse_version(s2)
-            assert p1 < p2, (s1, s2, p1, p2)
-
-        c('2.1', '2.1.1')
-        c('2a1', '2b0')
-        c('2a1', '2.1')
-        c('2.3a1', '2.3')
-        c('2.1-1', '2.1-2')
-        c('2.1-1', '2.1.1')
-        c('2.1', '2.1post4')
-        c('2.1a0-20040501', '2.1')
-        c('1.1', '02.1')
-        c('3.2', '3.2.post0')
-        c('3.2post1', '3.2post2')
-        c('0.4', '4.0')
-        c('0.0.4', '0.4.0')
-        c('0post1', '0.4post1')
-        c('2.1.0-rc1', '2.1.0')
-        c('2.1dev', '2.1a0')
-
-        torture = """
+    torture = """
         0.80.1-3 0.80.1-2 0.80.1-1 0.79.9999+0.80.0pre4-1
         0.79.9999+0.80.0pre2-3 0.79.9999+0.80.0pre2-2
         0.77.2-1 0.77.1-1 0.77.0-1
-        """.split()
-
-        for p, v1 in enumerate(torture):
-            for v2 in torture[p + 1:]:
-                c(v2, v1)
-
-    def testVersionBuildout(self):
         """
-        Buildout has a function in it's bootstrap.py that inspected the return
-        value of parse_version. The new parse_version returns a Version class
-        which needs to support this behavior, at least for now.
-        """
-        def buildout(parsed_version):
-            _final_parts = '*final-', '*final'
 
-            def _final_version(parsed_version):
-                for part in parsed_version:
-                    if (part[:1] == '*') and (part not in _final_parts):
-                        return False
-                return True
-            return _final_version(parsed_version)
-
-        assert buildout(parse_version("1.0"))
-        assert not buildout(parse_version("1.0a1"))
-
-    def testVersionIndexable(self):
-        """
-        Some projects were doing things like parse_version("v")[0], so we'll
-        support indexing the same as we support iterating.
-        """
-        assert parse_version("1.0")[0] == "00000001"
-
-    def testVersionTupleSort(self):
-        """
-        Some projects expected to be able to sort tuples against the return
-        value of parse_version. So again we'll add a warning enabled shim to
-        make this possible.
-        """
-        assert parse_version("1.0") < tuple(parse_version("2.0"))
-        assert parse_version("1.0") <= tuple(parse_version("2.0"))
-        assert parse_version("1.0") == tuple(parse_version("1.0"))
-        assert parse_version("3.0") > tuple(parse_version("2.0"))
-        assert parse_version("3.0") >= tuple(parse_version("2.0"))
-        assert parse_version("3.0") != tuple(parse_version("2.0"))
-        assert not (parse_version("3.0") != tuple(parse_version("3.0")))
+    @pytest.mark.parametrize(
+        ['lower', 'upper'],
+        [
+            ('2.1', '2.1.1'),
+            ('2a1', '2b0'),
+            ('2a1', '2.1'),
+            ('2.3a1', '2.3'),
+            ('2.1-1', '2.1-2'),
+            ('2.1-1', '2.1.1'),
+            ('2.1', '2.1post4'),
+            ('2.1a0-20040501', '2.1'),
+            ('1.1', '02.1'),
+            ('3.2', '3.2.post0'),
+            ('3.2post1', '3.2post2'),
+            ('0.4', '4.0'),
+            ('0.0.4', '0.4.0'),
+            ('0post1', '0.4post1'),
+            ('2.1.0-rc1', '2.1.0'),
+            ('2.1dev', '2.1a0'),
+        ] + list(pairwise(reversed(torture.split()))),
+    )
+    def testVersionOrdering(self, lower, upper):
+        assert parse_version(lower) < parse_version(upper)
 
     def testVersionHashable(self):
         """
@@ -733,7 +777,7 @@ class TestNamespaces:
 
     ns_str = "__import__('pkg_resources').declare_namespace(__name__)\n"
 
-    @pytest.yield_fixture
+    @pytest.fixture
     def symlinked_tmpdir(self, tmpdir):
         """
         Where available, return the tempdir as a symlink,
@@ -751,7 +795,7 @@ class TestNamespaces:
         finally:
             os.unlink(link_name)
 
-    @pytest.yield_fixture(autouse=True)
+    @pytest.fixture(autouse=True)
     def patched_path(self, tmpdir):
         """
         Patch sys.path to include the 'site-pkgs' dir. Also
@@ -790,10 +834,12 @@ class TestNamespaces:
             pkg2.ensure_dir()
             (pkg1 / '__init__.py').write_text(self.ns_str, encoding='utf-8')
             (pkg2 / '__init__.py').write_text(self.ns_str, encoding='utf-8')
-        import pkg1
+        with pytest.warns(DeprecationWarning, match="pkg_resources.declare_namespace"):
+            import pkg1
         assert "pkg1" in pkg_resources._namespace_packages
         # attempt to import pkg2 from site-pkgs2
-        import pkg1.pkg2
+        with pytest.warns(DeprecationWarning, match="pkg_resources.declare_namespace"):
+            import pkg1.pkg2
         # check the _namespace_packages dict
         assert "pkg1.pkg2" in pkg_resources._namespace_packages
         assert pkg_resources._namespace_packages["pkg1"] == ["pkg1.pkg2"]
@@ -831,10 +877,12 @@ class TestNamespaces:
             subpkg = nspkg / 'subpkg'
             subpkg.ensure_dir()
             (nspkg / '__init__.py').write_text(self.ns_str, encoding='utf-8')
-            (subpkg / '__init__.py').write_text(vers_str % number, encoding='utf-8')
+            (subpkg / '__init__.py').write_text(
+                vers_str % number, encoding='utf-8')
 
-        import nspkg.subpkg
-        import nspkg
+        with pytest.warns(DeprecationWarning, match="pkg_resources.declare_namespace"):
+            import nspkg.subpkg
+            import nspkg
         expected = [
             str(site.realpath() / 'nspkg')
             for site in site_dirs
