@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """sdist tests"""
 
 import contextlib
@@ -8,6 +7,7 @@ import sys
 import tempfile
 import itertools
 import io
+import logging
 from distutils import log
 from distutils.errors import DistutilsTemplateError
 
@@ -18,6 +18,9 @@ from setuptools.tests.textwrap import DALS
 import pytest
 
 
+IS_PYPY = '__pypy__' in sys.builtin_module_names
+
+
 def make_local_path(s):
     """Converts '/' in a string to os.sep"""
     return s.replace('/', os.sep)
@@ -321,41 +324,29 @@ class TestFileListTest(TempDirTestCase):
     to ensure setuptools' version of FileList keeps parity with distutils.
     """
 
-    def setup_method(self, method):
-        super(TestFileListTest, self).setup_method(method)
-        self.threshold = log.set_threshold(log.FATAL)
-        self._old_log = log.Log._log
-        log.Log._log = self._log
-        self.logs = []
+    @pytest.fixture(autouse=os.getenv("SETUPTOOLS_USE_DISTUTILS") == "stdlib")
+    def _compat_record_logs(self, monkeypatch, caplog):
+        """Account for stdlib compatibility"""
+        def _log(_logger, level, msg, args):
+            exc = sys.exc_info()
+            rec = logging.LogRecord("distutils", level, "", 0, msg, args, exc)
+            caplog.records.append(rec)
 
-    def teardown_method(self, method):
-        log.set_threshold(self.threshold)
-        log.Log._log = self._old_log
-        super(TestFileListTest, self).teardown_method(method)
+        monkeypatch.setattr(log.Log, "_log", _log)
 
-    def _log(self, level, msg, args):
-        if level not in (log.DEBUG, log.INFO, log.WARN, log.ERROR, log.FATAL):
-            raise ValueError('%s wrong log level' % str(level))
-        self.logs.append((level, msg, args))
+    def get_records(self, caplog, *levels):
+        return [r for r in caplog.records if r.levelno in levels]
 
-    def get_logs(self, *levels):
-        def _format(msg, args):
-            if len(args) == 0:
-                return msg
-            return msg % args
-        return [_format(msg, args) for level, msg, args
-                in self.logs if level in levels]
+    def assertNoWarnings(self, caplog):
+        assert self.get_records(caplog, log.WARN) == []
+        caplog.clear()
 
-    def clear_logs(self):
-        self.logs = []
-
-    def assertNoWarnings(self):
-        assert self.get_logs(log.WARN) == []
-        self.clear_logs()
-
-    def assertWarnings(self):
-        assert len(self.get_logs(log.WARN)) > 0
-        self.clear_logs()
+    def assertWarnings(self, caplog):
+        if IS_PYPY and not caplog.records:
+            pytest.xfail("caplog checks may not work well in PyPy")
+        else:
+            assert len(self.get_records(caplog, log.WARN)) > 0
+            caplog.clear()
 
     def make_files(self, files):
         for file in files:
@@ -472,7 +463,8 @@ class TestFileListTest(TempDirTestCase):
             else:
                 assert False, "Should have thrown an error"
 
-    def test_include(self):
+    def test_include(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # include
         file_list = FileList()
@@ -481,14 +473,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('include *.py')
         file_list.sort()
         assert file_list.files == ['a.py']
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('include *.rb')
         file_list.sort()
         assert file_list.files == ['a.py']
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_exclude(self):
+    def test_exclude(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # exclude
         file_list = FileList()
@@ -497,14 +490,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('exclude *.py')
         file_list.sort()
         assert file_list.files == ['b.txt', ml('d/c.py')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('exclude *.rb')
         file_list.sort()
         assert file_list.files == ['b.txt', ml('d/c.py')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_global_include(self):
+    def test_global_include(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # global-include
         file_list = FileList()
@@ -513,14 +507,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('global-include *.py')
         file_list.sort()
         assert file_list.files == ['a.py', ml('d/c.py')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('global-include *.rb')
         file_list.sort()
         assert file_list.files == ['a.py', ml('d/c.py')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_global_exclude(self):
+    def test_global_exclude(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # global-exclude
         file_list = FileList()
@@ -529,14 +524,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('global-exclude *.py')
         file_list.sort()
         assert file_list.files == ['b.txt']
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('global-exclude *.rb')
         file_list.sort()
         assert file_list.files == ['b.txt']
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_recursive_include(self):
+    def test_recursive_include(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # recursive-include
         file_list = FileList()
@@ -545,14 +541,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('recursive-include d *.py')
         file_list.sort()
         assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('recursive-include e *.py')
         file_list.sort()
         assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_recursive_exclude(self):
+    def test_recursive_exclude(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # recursive-exclude
         file_list = FileList()
@@ -561,14 +558,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('recursive-exclude d *.py')
         file_list.sort()
         assert file_list.files == ['a.py', ml('d/c.txt')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('recursive-exclude e *.py')
         file_list.sort()
         assert file_list.files == ['a.py', ml('d/c.txt')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_graft(self):
+    def test_graft(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # graft
         file_list = FileList()
@@ -577,14 +575,15 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('graft d')
         file_list.sort()
         assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('graft e')
         file_list.sort()
         assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
 
-    def test_prune(self):
+    def test_prune(self, caplog):
+        caplog.set_level(logging.DEBUG)
         ml = make_local_path
         # prune
         file_list = FileList()
@@ -593,9 +592,9 @@ class TestFileListTest(TempDirTestCase):
         file_list.process_template_line('prune d')
         file_list.sort()
         assert file_list.files == ['a.py', ml('f/f.py')]
-        self.assertNoWarnings()
+        self.assertNoWarnings(caplog)
 
         file_list.process_template_line('prune e')
         file_list.sort()
         assert file_list.files == ['a.py', ml('f/f.py')]
-        self.assertWarnings()
+        self.assertWarnings(caplog)
